@@ -1,18 +1,29 @@
-#===============================================================================
-# Copyright 2007 Matt Chaput
-# 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-# 
-#    http://www.apache.org/licenses/LICENSE-2.0
-# 
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-#===============================================================================
+# Copyright 2007 Matt Chaput. All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
+#
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY MATT CHAPUT ``AS IS'' AND ANY EXPRESS OR
+# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
+# EVENT SHALL MATT CHAPUT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
+# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
+# The views and conclusions contained in the software and documentation are
+# those of the authors and should not be interpreted as representing official
+# policies, either expressed or implied, of Matt Chaput.
 
 """Miscellaneous utility functions and classes.
 """
@@ -23,50 +34,16 @@ import re
 import sys
 import time
 from array import array
+from bisect import insort, bisect_left, bisect_right
 from copy import copy
 from functools import wraps
-from math import log
 from struct import pack, unpack
 from threading import Lock
 
-from whoosh.system import IS_LITTLE
-
-
-try:
-    from itertools import permutations
-except ImportError:
-    # This function was only added to itertools in 2.6...
-    def permutations(iterable, r=None):
-        pool = tuple(iterable)
-        n = len(pool)
-        r = n if r is None else r
-        if r > n:
-            return
-        indices = range(n)
-        cycles = range(n, n - r, -1)
-        yield tuple(pool[i] for i in indices[:r])
-        while n:
-            for i in reversed(range(r)):
-                cycles[i] -= 1
-                if cycles[i] == 0:
-                    indices[i:] = indices[i + 1:] + indices[i:i + 1]
-                    cycles[i] = n - i
-                else:
-                    j = cycles[i]
-                    indices[i], indices[-j] = indices[-j], indices[i]
-                    yield tuple(pool[i] for i in indices[:r])
-                    break
-            else:
-                return
-
-
-try:
-    from operator import methodcaller
-except ImportError:
-    def methodcaller(name, *args, **kwargs):
-        def caller(obj):
-            return getattr(obj, name)(*args, **kwargs)
-        return caller
+from whoosh.compat import xrange, u, b, string_type
+from whoosh.compat import array_tobytes
+from whoosh.system import pack_ushort_le, pack_uint_le
+from whoosh.system import unpack_ushort_le, unpack_uint_le
 
 
 if sys.platform == 'win32':
@@ -78,30 +55,20 @@ else:
 # Note: these functions return a tuple of (text, length), so when you call
 # them, you have to add [0] on the end, e.g. str = utf8encode(unicode)[0]
 
-utf8encode = codecs.getencoder("utf_8")
-utf8decode = codecs.getdecoder("utf_8")
+utf8encode = codecs.getencoder("utf-8")
+utf8decode = codecs.getdecoder("utf-8")
+
+#utf16encode = codecs.getencoder("utf-16-be")
+#utf16decode = codecs.getdecoder("utf-16-be")
+#utf32encode = codecs.getencoder("utf-32-be")
+#utf32decode = codecs.getdecoder("utf-32-be")
 
 
 # Functions
 
-def array_to_string(a):
-    if IS_LITTLE:
-        a = copy(a)
-        a.byteswap()
-    return a.tostring()
-
-
-def string_to_array(typecode, s):
-    a = array(typecode)
-    a.fromstring(s)
-    if IS_LITTLE:
-        a.byteswap()
-    return a
-
-
 def make_binary_tree(fn, args, **kwargs):
     """Takes a function/class that takes two positional arguments and a list of
-    arguments and returns a binary tree of instances.
+    arguments and returns a binary tree of results/instances.
     
     >>> make_binary_tree(UnionMatcher, [matcher1, matcher2, matcher3])
     UnionMatcher(matcher1, UnionMatcher(matcher2, matcher3))
@@ -109,18 +76,35 @@ def make_binary_tree(fn, args, **kwargs):
     Any keyword arguments given to this function are passed to the class
     initializer.
     """
-    
+
     count = len(args)
     if not count:
         raise ValueError("Called make_binary_tree with empty list")
     elif count == 1:
         return args[0]
-    
+
     half = count // 2
     return fn(make_binary_tree(fn, args[:half], **kwargs),
               make_binary_tree(fn, args[half:], **kwargs), **kwargs)
 
 
+def make_weighted_tree(fn, ls, **kwargs):
+    """Takes a function/class that takes two positional arguments and a list of
+    (weight, argument) tuples and returns a huffman-like weighted tree of
+    results/instances.
+    """
+
+    if not ls:
+        raise ValueError("Called make_weighted_tree with empty list")
+
+    ls.sort()
+    while len(ls) > 1:
+        a = ls.pop(0)
+        b = ls.pop(0)
+        insort(ls, (a[0] + b[0], fn(a[1], b[1])))
+    return ls[0][1]
+
+
 # Varint cache
 
 # Build a cache of the varint byte sequences for the first N integers, so we
@@ -128,12 +112,12 @@ def make_binary_tree(fn, args, **kwargs):
 # noticeable difference.
 
 def _varint(i):
-    s = ""
+    a = array("B")
     while (i & ~0x7F) != 0:
-        s += chr((i & 0x7F) | 0x80)
+        a.append((i & 0x7F) | 0x80)
         i = i >> 7
-    s += chr(i)
-    return s
+    a.append(i)
+    return array_tobytes(a)
 
 
 _varint_cache_size = 512
@@ -167,7 +151,7 @@ def varint_to_int(vi):
 def signed_varint(i):
     """Zig-zag encodes a signed integer into a varint.
     """
-    
+
     if i >= 0:
         return varint(i << 1)
     return varint((i << 1) ^ (~0))
@@ -176,7 +160,7 @@ def signed_varint(i):
 def decode_signed_varint(i):
     """Zig-zag decodes an integer value.
     """
-    
+
     if not i & 1:
         return i >> 1
     return (i >> 1) ^ (~0)
@@ -234,20 +218,22 @@ def float_to_byte(value, mantissabits=5, zeroexp=2):
         # Map negative numbers and 0 to 0
         # Map underflow to next smallest non-zero number
         if bits <= 0:
-            return chr(0)
+            result = chr(0)
         else:
-            return chr(1)
+            result = chr(1)
     elif smallfloat >= fzero + 0x100:
         # Map overflow to largest number
-        return chr(255)
+        result = chr(255)
     else:
-        return chr(smallfloat - fzero)
+        result = chr(smallfloat - fzero)
+    return b(result)
 
 
 def byte_to_float(b, mantissabits=5, zeroexp=2):
     """Decodes a floating point number stored in a single byte.
     """
-    b = ord(b)
+    if type(b) is not int:
+        b = ord(b)
     if b == 0:
         return 0.0
 
@@ -258,31 +244,74 @@ def byte_to_float(b, mantissabits=5, zeroexp=2):
 
 # Length-to-byte approximation functions
 
+# Old implementation:
+
+#def length_to_byte(length):
+#    """Returns a logarithmic approximation of the given number, in the range
+#    0-255. The approximation has high precision at the low end (e.g.
+#    1 -> 0, 2 -> 1, 3 -> 2 ...) and low precision at the high end. Numbers
+#    equal to or greater than 108116 all approximate to 255.
+#
+#    This is useful for storing field lengths, where the general case is small
+#    documents and very large documents are more rare.
+#    """
+#
+#    # This encoding formula works up to 108116 -> 255, so if the length is
+#    # equal to or greater than that limit, just return 255.
+#    if length >= 108116:
+#        return 255
+#
+#    # The parameters of this formula where chosen heuristically so that low
+#    # numbers would approximate closely, and the byte range 0-255 would cover
+#    # a decent range of document lengths (i.e. 1 to ~100000).
+#    return int(round(log((length / 27.0) + 1, 1.033)))
+#def _byte_to_length(n):
+#    return int(round((pow(1.033, n) - 1) * 27))
+#_b2l_cache = array("i", (_byte_to_length(i) for i in xrange(256)))
+#byte_to_length = _b2l_cache.__getitem__
+
+# New implementation
+
+# Instead of computing the actual formula to get the byte for any given length,
+# precompute the length associated with each byte, and use bisect to find the
+# nearest value. This gives quite a large speed-up.
+#
+# Note that this does not give all the same answers as the old, "real"
+# implementation since this implementation always "rounds down" (thanks to the
+# bisect_left) while the old implementation would "round up" or "round down"
+# depending on the input. Since this is a fairly gross approximation anyway,
+# I don't think it matters much.
+
+# Values generated using the formula from the "old" implementation above
+_length_byte_cache = array('i', [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 14,
+16, 17, 18, 20, 21, 23, 25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 45, 47, 49, 52,
+54, 57, 60, 63, 66, 69, 72, 75, 79, 82, 86, 89, 93, 97, 101, 106, 110, 114,
+119, 124, 129, 134, 139, 145, 150, 156, 162, 169, 175, 182, 189, 196, 203, 211,
+219, 227, 235, 244, 253, 262, 271, 281, 291, 302, 313, 324, 336, 348, 360, 373,
+386, 399, 414, 428, 443, 459, 475, 491, 508, 526, 544, 563, 583, 603, 623, 645,
+667, 690, 714, 738, 763, 789, 816, 844, 873, 903, 933, 965, 998, 1032, 1066,
+1103, 1140, 1178, 1218, 1259, 1302, 1345, 1391, 1438, 1486, 1536, 1587, 1641,
+1696, 1753, 1811, 1872, 1935, 1999, 2066, 2135, 2207, 2280, 2356, 2435, 2516,
+2600, 2687, 2777, 2869, 2965, 3063, 3165, 3271, 3380, 3492, 3608, 3728, 3852,
+3980, 4112, 4249, 4390, 4536, 4686, 4842, 5002, 5168, 5340, 5517, 5700, 5889,
+6084, 6286, 6494, 6709, 6932, 7161, 7398, 7643, 7897, 8158, 8428, 8707, 8995,
+9293, 9601, 9918, 10247, 10586, 10936, 11298, 11671, 12057, 12456, 12868,
+13294, 13733, 14187, 14656, 15141, 15641, 16159, 16693, 17244, 17814, 18403,
+19011, 19640, 20289, 20959, 21652, 22367, 23106, 23869, 24658, 25472, 26314,
+27183, 28081, 29009, 29967, 30957, 31979, 33035, 34126, 35254, 36418, 37620,
+38863, 40146, 41472, 42841, 44256, 45717, 47227, 48786, 50397, 52061, 53780,
+55556, 57390, 59285, 61242, 63264, 65352, 67510, 69739, 72041, 74419, 76876,
+79414, 82035, 84743, 87541, 90430, 93416, 96499, 99684, 102975, 106374])
+
+
 def length_to_byte(length):
-    """Returns a logarithmic approximation of the given number, in the range
-    0-255. The approximation has high precision at the low end (e.g.
-    1 -> 0, 2 -> 1, 3 -> 2 ...) and low precision at the high end. Numbers
-    equal to or greater than 108116 all approximate to 255.
-    
-    This is useful for storing field lengths, where the general case is small
-    documents and very large documents are more rare.
-    """
-    
-    # This encoding formula works up to 108116 -> 255, so if the length is
-    # equal to or greater than that limit, just return 255.
-    if length >= 108116:
+    if length is None:
+        return 0
+    if length >= 106374:
         return 255
-    
-    # The parameters of this formula where chosen heuristically so that low
-    # numbers would approximate closely, and the byte range 0-255 would cover
-    # a decent range of document lengths (i.e. 1 to ~100000).
-    return int(round(log((length / 27.0) + 1, 1.033)))
+    else:
+        return bisect_left(_length_byte_cache, length)
 
-
-def _byte_to_length(n):
-    return int(round((pow(1.033, n) - 1) * 27))
-
-_length_byte_cache = array("i", (_byte_to_length(i) for i in xrange(256)))
 byte_to_length = _length_byte_cache.__getitem__
 
 
@@ -295,12 +324,11 @@ def first_diff(a, b):
     byte.
     """
 
-    i = -1
+    i = 0
     for i in xrange(0, len(a)):
-        if a[i] != b[1]:
-            return i
-        if i == 255:
-            return i
+        if a[i] != b[i] or i == 255:
+            break
+    return i
 
 
 def prefix_encode(a, b):
@@ -318,7 +346,7 @@ def prefix_encode_all(ls):
     as UTF-8.
     """
 
-    last = u''
+    last = u('')
     for w in ls:
         i = first_diff(last, w)
         yield chr(i) + w[i:].encode("utf8")
@@ -329,7 +357,7 @@ def prefix_decode_all(ls):
     """Decompresses a list of strings compressed by prefix_encode().
     """
 
-    last = u''
+    last = u('')
     for w in ls:
         i = ord(w[0])
         decoded = last[:i] + w[1:].decode("utf8")
@@ -393,13 +421,13 @@ def protected(func):
             return func(self, *args, **kwargs)
 
     return protected_wrapper
-    
+
 
 def synchronized(func):
     """Decorator for storage-access methods, which synchronizes on a threading
     lock. The parent object must have 'is_closed' and '_sync_lock' attributes.
     """
-    
+
     @wraps(func)
     def synchronized_wrapper(self, *args, **kwargs):
         with self._sync_lock:
@@ -411,9 +439,9 @@ def synchronized(func):
 def unbound_cache(func):
     """Caching decorator with an unbounded cache size.
     """
-    
+
     cache = {}
-    
+
     @wraps(func)
     def caching_wrapper(*args):
         try:
@@ -422,11 +450,70 @@ def unbound_cache(func):
             result = func(*args)
             cache[args] = result
             return result
-        
+
     return caching_wrapper
 
 
 def lru_cache(maxsize=100):
+    """Double-barrel least-recently-used cache decorator. This is a simple
+    LRU algorithm that keeps a primary and secondary dict. Keys are checked
+    in the primary dict, and then the secondary. Once the primary dict fills
+    up, the secondary dict is cleared and the two dicts are swapped.
+    
+    This function duplicates (more-or-less) the protocol of the
+    ``functools.lru_cache`` decorator in the Python 3.2 standard library.
+
+    Arguments to the cached function must be hashable.
+
+    View the cache statistics named tuple (hits, misses, maxsize, currsize)
+    with f.cache_info().  Clear the cache and statistics with f.cache_clear().
+    Access the underlying function with f.__wrapped__.
+    """
+
+    def decorating_function(user_function):
+        # Cache1, Cache2, Pointer, Hits, Misses
+        stats = [{}, {}, 0, 0, 0]
+
+        @wraps(user_function)
+        def wrapper(*args):
+            ptr = stats[2]
+            a = stats[ptr]
+            b = stats[not ptr]
+            key = args
+
+            if key in a:
+                stats[3] += 1  # Hit
+                return a[key]
+            elif key in b:
+                stats[3] += 1  # Hit
+                return b[key]
+            else:
+                stats[4] += 1  # Miss
+                result = user_function(*args)
+                a[key] = result
+                if len(a) >= maxsize:
+                    stats[2] = not ptr
+                    b.clear()
+                return result
+
+        def cache_info():
+            """Report cache statistics"""
+            return (stats[3], stats[4], maxsize, len(stats[0]) + len(stats[1]))
+
+        def cache_clear():
+            """Clear the cache and cache statistics"""
+            stats[0].clear()
+            stats[1].clear()
+            stats[3] = stats[4] = 0
+
+        wrapper.cache_info = cache_info
+        wrapper.cache_clear = cache_clear
+
+        return wrapper
+    return decorating_function
+
+
+def clockface_lru_cache(maxsize=100):
     """Least-recently-used cache decorator.
 
     This function duplicates (more-or-less) the protocol of the
@@ -442,19 +529,19 @@ def lru_cache(maxsize=100):
     with f.cache_info().  Clear the cache and statistics with f.cache_clear().
     Access the underlying function with f.__wrapped__.
     """
-    
+
     def decorating_function(user_function):
 
-        stats = [0, 0, 0]  # hits, misses
+        stats = [0, 0, 0]  # hits, misses, hand
         data = {}
-        
+
         if maxsize:
             # The keys at each point on the clock face
             clock_keys = [None] * maxsize
             # The "referenced" bits at each point on the clock face
             clock_refs = array("B", (0 for _ in xrange(maxsize)))
             lock = Lock()
-            
+
             @wraps(user_function)
             def wrapper(*args):
                 key = args
@@ -500,7 +587,7 @@ def lru_cache(maxsize=100):
                         # Record a cache miss
                         stats[1] += 1
                 return result
-        
+
         else:
             @wraps(user_function)
             def wrapper(*args):
@@ -539,11 +626,12 @@ def find_object(name, blacklist=None, whitelist=None):
     >>> find_object("whoosh.analysis.StopFilter")
     <class 'whoosh.analysis.StopFilter'>
     """
-    
+
     if blacklist:
         for pre in blacklist:
             if name.startswith(pre):
-                raise TypeError("%r: can't instantiate names starting with %r" % (name, pre))
+                raise TypeError("%r: can't instantiate names starting with %r"
+                                % (name, pre))
     if whitelist:
         passes = False
         for pre in whitelist:
@@ -552,27 +640,29 @@ def find_object(name, blacklist=None, whitelist=None):
                 break
         if not passes:
             raise TypeError("Can't instantiate %r" % name)
-    
+
     lastdot = name.rfind(".")
-    
+
     assert lastdot > -1, "Name %r must be fully qualified" % name
     modname = name[:lastdot]
     clsname = name[lastdot + 1:]
-    
+
     mod = __import__(modname, fromlist=[clsname])
     cls = getattr(mod, clsname)
     return cls
 
 
-
-
-
-
-
-
-
-
-
+def rcompile(pattern, flags=0, verbose=False):
+    """A wrapper for re.compile that checks whether "pattern" is a regex object
+    or a string to be compiled, and automatically adds the re.UNICODE flag.
+    """
+
+    if not isinstance(pattern, string_type):
+        # If it's not a string, assume it's already a compiled pattern
+        return pattern
+    if verbose:
+        flags |= re.VERBOSE
+    return re.compile(pattern, re.UNICODE | flags)
 
 
 
